@@ -2,40 +2,45 @@ package Sort::SQL;
 
 use strict;
 use warnings;
+use vars qw( $VERSION );    # for version 5.005...
 
-use vars qw( $VERSION );
+$VERSION = '0.08';
 
-$VERSION = '0.03';
+my $debug = $ENV{PERL_DEBUG} || 0;
 
-sub string2array
-{
+sub parse {
     my $class = shift;
     my $order = shift || '';    # will return empty array
 
-    my @s = split(m/\ +/, $order);
-    my @p;
-    my %o;
+    my @s = split( m/[\ ,]+/, $order );
+    my @pairs;
 
-    while (my ($prop, $dir) = splice(@s, 0, 2))
-    {
-        if (!defined $dir)
-        {
+    while ( my ( $prop, $dir ) = splice( @s, 0, 2 ) ) {
+
+        $debug and warn sprintf( "prop='%s'  dir='%s'\n", $prop, $dir || '' );
+
+        next if $prop =~ m/[^\.\w]/;    # avoid sql injection
+
+        if ( !defined $dir ) {
             $dir = 'ASC';
         }
-        elsif ($dir !~ m/^(ASC|DESC)$/i)
-        {
-            unshift(@s, $dir);
+        elsif ( $dir !~ m/^(ASC|DESC)$/i ) {
+            unshift( @s, $dir );
             $dir = 'ASC';
         }
 
-        $o{$prop} = $dir;
-        push(@p, $prop);
+        push @pairs, [ $prop => uc($dir) ];
     }
 
+    return \@pairs;
+}
+
+sub string2array {
+    my $pairs = shift->parse(@_);
     return [
         map {
-            { $_ => $o{$_} }
-          } @p
+            { $_->[0] => $_->[1] }
+            } @$pairs
     ];
 }
 
@@ -49,28 +54,35 @@ Sort::SQL - manipulate SQL sort strings
 =head1 SYNOPSIS
 
   use Sort::SQL;
-  
-  my $array = Sort::SQL->string2array('foo ASC bar DESC bing');
-  # $array == [ { foo => 'ASC' }, { bar => 'DESC' }, { bing  => 'ASC' } ]
-  
 
+  my $array_of_arrays = Sort::SQL->parse('foo ASC bar DESC bing');
+  # $array_of_arrays == [ [ 'foo', 'ASC' ], [ 'bar', 'DESC' ], [ 'bing', 'ASC' ] ]
+  
+  my $array_of_hashes = Sort::SQL->string2array('foo asc bar DESC bing');
+  # $array_of_hashes == [ { foo => 'ASC' }, { bar => 'DESC' }, { bing  => 'ASC' } ]
+  
 =head1 DESCRIPTION
 
-Sort::SQL is so simple it almost doesn't deserve to be on CPAN. But since I kept
-finding myself copy/pasting this method into multiple places, I finally figured
+Sort::SQL is so simple it almost doesn't deserve to be on CPAN. 
+But since I kept finding myself copy/pasting this method 
+into multiple places, I finally figured
 it deserved to live in its own class.
 
 =head1 METHODS
 
-Sort::SQL implements one method: string2array().
+Sort::SQL implements these methods, each of which
+take a scalar string of the SQL C<ORDER BY> syntax and return
+a data structure.
+
+=head2 parse( I<sql sort string> )
+
+Returns I<sql sort string> as an arrayref of arrayrefs.
 
 =head2 string2array( I<sql sort string> )
 
-Takes a scalar string of the SQL C<ORDER BY> syntax and turns it into
-an array of key/value pair hashrefs.
-
-I use this method frequently in my Template Toolkit applications, where I want
-to be able to create re-sortable table columns. I presume there are many other uses.
+Returns I<sql sort string> as an arrayref of hashrefs. This is for
+backwards compat only -- parse() is a little faster and more
+usable.
 
 =head1 EXAMPLE
 
@@ -82,7 +94,7 @@ as a param in the URI.
 In my controller code I do:
 
  my $sort_order = $c->request->param('order');
- $c->stash->{search}->{order} = Sort::SQL->string2array( $sort_order );
+ $c->stash->{search}->{order} = Sort::SQL->parse( $sort_order );
  
 And then in my template:
 
@@ -90,8 +102,8 @@ And then in my template:
  
  FOREACH pair IN search.order;
 
-  column     = pair.each.0;
-  direction  = pair.each.1;
+  column     = pair.0;
+  direction  = pair.1;
 
   # toggle the sort direction for current sorted column
   # so the next browser request will reverse the sort
